@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Chemistry Utils
- * gccv/text.cc 
+ * gccv/text.cc
  *
- * Copyright (C) 2008-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2008-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -235,16 +235,16 @@ void TextLine::DrawDecorations (cairo_t *cr, bool is_vector)
 			y -= width;
 			cairo_move_to (cr, xstart, y);
 			cairo_line_to (cr, xend, y);
-			break;	
+			break;
 		case TextDecorationLow:
 			y += width;
 			cairo_move_to (cr, xstart, y);
 			cairo_line_to (cr, xend, y);
-			break;	
+			break;
 		case TextDecorationMedium:
 			cairo_move_to (cr, xstart, y);
 			cairo_line_to (cr, xend, y);
-			break;	
+			break;
 		case TextDecorationDouble:
 			y += width;
 			cairo_move_to (cr, xstart, y);
@@ -252,9 +252,9 @@ void TextLine::DrawDecorations (cairo_t *cr, bool is_vector)
 			y -= 2 * width;
 			cairo_move_to (cr, xstart, y);
 			cairo_line_to (cr, xend, y);
-			break;	
+			break;
 		default:
-			break;	
+			break;
 		}
 		cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (color));
 		cairo_set_line_width (cr, width);
@@ -494,7 +494,7 @@ void Text::SetPosition (double x, double y)
 	case AnchorSouth:
 	case AnchorSouthWest:
 	case AnchorSouthEast:
-		yr = m_y - m_Height + m_Padding;
+		yr = m_y - m_Height - m_Padding;
 		break;
 	}
 	Rectangle::SetPosition (xr, yr, w, h);
@@ -541,17 +541,17 @@ void Text::Draw (cairo_t *cr, bool is_vector) const
 	case AnchorCenter:
 	case AnchorWest:
 	case AnchorEast:
-		starty = m_y - m_Height / 2. - m_LineOffset;
+		starty = m_y - m_Height / 2.;
 		break;
 	case AnchorNorth:
 	case AnchorNorthWest:
 	case AnchorNorthEast:
-		starty = m_y - m_LineOffset;
+		starty = m_y;
 		break;
 	case AnchorSouth:
 	case AnchorSouthWest:
 	case AnchorSouthEast:
-		starty = m_y - m_Height - m_LineOffset;
+		starty = m_y - m_Height;
 		break;
 	}
 	if (m_CurPos >= m_StartSel) {
@@ -767,17 +767,17 @@ void Text::GetBounds (Rect *ink, Rect *logical)
 	case AnchorCenter:
 	case AnchorWest:
 	case AnchorEast:
-		starty = m_y - m_Height / 2. - m_LineOffset;
+		starty = m_y - m_Height / 2./* + m_LineOffset*/;
 		break;
 	case AnchorNorth:
 	case AnchorNorthWest:
 	case AnchorNorthEast:
-		starty = m_y - m_LineOffset;
+		starty = m_y/*  + m_LineOffset*/;
 		break;
 	case AnchorSouth:
 	case AnchorSouthWest:
 	case AnchorSouthEast:
-		starty = m_y - m_Height - m_LineOffset;
+		starty = m_y - m_Height/* +  + m_LineOffset*/;
 		break;
 	}
 	if (ink) {
@@ -843,6 +843,12 @@ void Text::DeleteTextTag (TextTag *tag, bool rebuild_attributes)
 		RebuildAttributes ();
 }
 
+void Text::ClearTags ()
+{
+	m_Tags.clear ();
+	RebuildAttributes ();
+}
+
 void Text::SetCurTagList (TextTagList *l)
 {
 	if (m_CurTags)
@@ -868,7 +874,7 @@ void Text::ApplyTagsToSelection (TextTagList const *l)
 	// store tags to potentially insert into a vector
 	std::vector <TextTag *> new_tags (TextTag::MaxTag);
 	// fist set all vectors to NULL
-	for (int n = 0; n < MaxTag; n++) 
+	for (int n = 0; n < MaxTag; n++)
 		new_tags[n] = NULL;
 	// set the tags, unuseful ones will be reset to NULL later
 	for (j = l->begin (); j != jend; j++)
@@ -902,7 +908,7 @@ void Text::ApplyTagsToSelection (TextTagList const *l)
 		}
 	}
 	// Add new tags
-	for (int n = 0; n < TextTag::MaxTag; n++) 
+	for (int n = 0; n < TextTag::MaxTag; n++)
 		if (new_tags[n]) {
 			TextTag *tag = new_tags[n]->Duplicate ();
 			tag->SetStartIndex (start);
@@ -912,7 +918,7 @@ void Text::ApplyTagsToSelection (TextTagList const *l)
 			else
 				m_Tags.push_back (tag);
 		}
-	
+
 	// Add extra tags
 	iend = extra_tags.end ();
 	for (i = extra_tags.begin (); i != iend; i++)
@@ -940,7 +946,7 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	if (length > 0) {
 		m_Text.erase (pos, length);
 		std::vector <TextTag *> borders (TextTag::MaxTag);
-		for (int n = 0; n < TextTag::MaxTag; n++) 
+		for (int n = 0; n < TextTag::MaxTag; n++)
 			borders[n] = NULL;
 		//TODO: update runs
 		for (i = m_Tags.begin (); i != iend; i++) {
@@ -1028,7 +1034,7 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	for (i = m_Tags.begin (); i != iend; i++) {
 		if ((*i)->GetEndIndex () < static_cast <unsigned> (pos))
 			continue;
-		if ((*i)->GetStartIndex () >= static_cast <unsigned> (pos)) {
+		if ((*i)->GetStartIndex () > static_cast <unsigned> (pos)) {
 			(*i)->SetStartIndex ((*i)->GetStartIndex () + nl);
 			(*i)->SetEndIndex ((*i)->GetEndIndex () + nl);
 		}
@@ -1061,7 +1067,7 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 		else
 			m_Tags.push_back (*i);
 	new_tags.clear (); // avoid destroying the tags
-	
+
 	iend = extra_tags.end ();
 	for (i = extra_tags.begin (); i != iend; i++) {
 		TextTag *tag = (*i)->Duplicate ();
@@ -1091,11 +1097,11 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 	}
 
 	switch (event->keyval) {
-	case GDK_Control_L:
-	case GDK_Control_R:
+	case GDK_KEY_Control_L:
+	case GDK_KEY_Control_R:
 		return false;
-	case GDK_Return:
-	case GDK_KP_Enter: {
+	case GDK_KEY_Return:
+	case GDK_KEY_KP_Enter: {
 		m_Text.insert (m_CurPos, "\n");
 		TextTag *tag = new NewLineTextTag ();
 		tag->SetStartIndex (m_CurPos++);
@@ -1108,14 +1114,14 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 			client->TextChanged (m_CurPos);
 		return true;
 	}
-	case GDK_Tab:
+	case GDK_KEY_Tab:
 		TextPrivate::OnCommit (m_ImContext, "\t", this);
 		if (client)
 			client->TextChanged (m_CurPos);
 		break;
 
 	/* MOVEMENT */
-	case GDK_Right:
+	case GDK_KEY_Right:
 		if (m_CurPos == m_Text.length ())
 			break;
 		if (event->state & GDK_CONTROL_MASK) {
@@ -1143,7 +1149,7 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 		if (client)
 			client->SelectionChanged (m_StartSel, m_CurPos);
 		break;
-	case GDK_Left:
+	case GDK_KEY_Left:
 		if (m_CurPos == 0)
 			break;
 		if (event->state & GDK_CONTROL_MASK) {
@@ -1171,42 +1177,42 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 		if (client)
 			client->SelectionChanged (m_StartSel, m_CurPos);
 		break;
-	case GDK_f:
+	case GDK_KEY_f:
 		/* TODO: write this code */
 		break;
-	case GDK_b:
+	case GDK_KEY_b:
 		/* TODO: write this code */
 		break;
-	case GDK_p:
+	case GDK_KEY_p:
 		if (!(event->state & GDK_CONTROL_MASK))
 			break;
-	case GDK_Up:
+	case GDK_KEY_Up:
 		/* TODO: write this code */
 		break;
-	case GDK_n:
+	case GDK_KEY_n:
 		if (!(event->state & GDK_CONTROL_MASK))
 			break;
-	case GDK_Down:
+	case GDK_KEY_Down:
 		/* TODO: write this code */
 		break;
-	case GDK_Home:
+	case GDK_KEY_Home:
 		/* TODO: write this code */
 		break;
-	case GDK_End:
+	case GDK_KEY_End:
 		/* TODO: write this code */
 		break;
-	case GDK_a:
+	case GDK_KEY_a:
 		/* TODO: write this code */
 		break;
-	case GDK_e:
+	case GDK_KEY_e:
 		/* TODO: write this code */
 		if (event->state & GDK_CONTROL_MASK) {
 		}
 		break;
 
 	/* DELETING TEXT */
-	case GDK_Delete:
-	case GDK_KP_Delete: {
+	case GDK_KEY_Delete:
+	case GDK_KEY_KP_Delete: {
 		if (m_CurPos != m_StartSel) {
 			ReplaceText (empty_st, MIN (m_CurPos, m_StartSel), abs (m_CurPos - m_StartSel));
 			if (client)
@@ -1223,10 +1229,10 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 			client->TextChanged (m_CurPos);
 		break;
 	}
-	case GDK_d:
+	case GDK_KEY_d:
 		/* TODO: write this code */
 		break;
-	case GDK_BackSpace: {
+	case GDK_KEY_BackSpace: {
 		if (m_CurPos != m_StartSel) {
 			ReplaceText (empty_st, MIN (m_CurPos, m_StartSel), abs (m_CurPos - m_StartSel));
 			if (client)
@@ -1243,19 +1249,19 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 			client->TextChanged (m_CurPos);
 		break;
 	}
-	case GDK_k:
+	case GDK_KEY_k:
 		if (event->state & GDK_CONTROL_MASK) {
 			/* delete from cursor to end of paragraph */
 			/* TODO: write this code */
 		}
 		break;
-	case GDK_u:
+	case GDK_KEY_u:
 		if (event->state & GDK_CONTROL_MASK) {
 			/* delete whole paragraph */
 			/* TODO: write this code */
 		}
 		break;
-	case GDK_backslash:
+	case GDK_KEY_backslash:
 		if (event->state & GDK_MOD1_MASK) {
 			/* delete all white spaces around the cursor */
 			/* TODO: write this code */
@@ -1661,4 +1667,13 @@ void Text::SetJustification (GtkJustification justification, bool emit_changed)
 	}
 }
 
+double Text::GetMaxLineHeight ()
+{
+	double res = 0.;
+	for (unsigned i = 0; i < m_LinesNumber; i++)
+		if (m_Lines[i].m_Height > res)
+			res = m_Lines[i].m_Height;
+	return res;
+}
+
 }
