@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -38,7 +38,7 @@
 #include <gccv/group.h>
 #include <gccv/text.h>
 #include <gccv/text-tag.h>
-#include <gcu/chain.h>
+#include <gcu/cycle.h>
 #include <gcu/element.h>
 #include <gcu/objprops.h>
 #include <gcugtk/ui-manager.h>
@@ -232,10 +232,11 @@ void Atom::Update ()
 	}
 	int nbonds = GetTotalBondsNumber ();
 	if (m_Valence > 0 && !m_Element->IsMetallic ()) {
-		m_nlp = (m_Element->GetValenceElectrons () - ((nbonds > m_Valence)? nbonds: m_Valence)) / 2;
-		if ((m_Charge > 0) && (m_nlp > 0)) m_nlp -= (m_Charge + 1) / 2;
+		m_nlp = m_Element->GetValenceElectrons () - ((nbonds > m_Valence)? nbonds: m_Valence);
+		if ((m_Charge > 0) && (m_nlp > 0)) m_nlp -= m_Charge;
 		else if (m_Charge < 0)
-			m_nlp -= m_Charge;
+			m_nlp -= (m_Element->GetValenceElectrons () < 4)? m_Charge: m_Charge * 2;
+		m_nlp /= 2;
 		if (m_nlp < nexplp) // Can this occur ?
 			m_nlp = nexplp;
 		else if (m_nlp > m_ValenceOrbitals - nbonds - nexplu) {
@@ -1513,17 +1514,47 @@ bool Atom::SetProperty (unsigned property, char const *value)
 	return  true;
 }
 
+std::string Atom::GetProperty (unsigned property) const
+{
+	std::ostringstream res;
+	switch (property) {
+	case GCU_PROP_TEXT_TEXT:
+		if (GetZ () != 6 || m_ShowSymbol) {
+			res << GetSymbol ();
+			if (m_nH > 0) {
+				res << "H";
+				if (m_nH > 1)
+					res << m_nH;
+			}
+		}
+		break;
+	case GCU_PROP_TEXT_POSITION:
+		if (GetZ () != 6 || m_ShowSymbol) {
+			gcu::Document const *doc = GetDocument ();
+			Theme *theme = static_cast < Document const * > (doc)->GetTheme ();
+			res << (m_x - m_width / 2. + theme->GetPadding () / theme->GetZoomFactor ()) / doc->GetScale ()
+				<< " " << (m_y + static_cast < Document const * > (doc)->GetView ()->GetCHeight ()) / doc->GetScale ();
+		}
+		break;
+	default:
+		return gcu::Atom::GetProperty (property);
+	}
+	return res.str ();
+}
+
 bool Atom::UpdateStereoBonds ()
 {
 	unsigned length[4]; // lengths before end or cycle 0 means cyclic bond
 	unsigned cycle_size[4]; // size of the nearest and largest attached cycle
 	unsigned cycle_pos[4]; // position of the cycle
+	unsigned is_cyclic[4];
 	Bond *bond[4];
 	double x[4], y[4];
 	std::list < unsigned > sorted;
 	std::list < unsigned >::iterator s, send;
+	unsigned cyclic_bonds = 0;
 	// we need to determine wich bonds should be considered as stereobonds
-	// the answer is that stereocenters shound not be bonded by a stereobond
+	// the answer is that stereocenters should not be bonded by a stereobond
 	// then the bond should not be cyclic
 	// then it shoud be in the shortest possible chain
 	// it should be not more that 90° from the nearest bond (which might also be
@@ -1536,16 +1567,39 @@ bool Atom::UpdateStereoBonds ()
 			length[i] = 0;
 			cycle_size[i] = 0;
 			cycle_pos[i] = 0;
+			is_cyclic[i] = false;
 			continue;
 		}
 		bond[i] = static_cast < Bond * > (GetBond (m_Bonded[i]));
+		is_cyclic[i] = bond[i]->IsCyclic ();
 		if (!bond[i]) // not everything has been loaded
 			return false;
 		// search if the bonded atom is a stereocenter
-		if (static_cast < Molecule * > (GetMolecule ())->AtomIsChiral (m_Bonded[i]) ||
-		    bond[i]->IsCyclic ()) {
+		if (static_cast < Molecule * > (GetMolecule ())->AtomIsChiral (m_Bonded[i])) {
 			length[i] = cycle_pos[i] = G_MAXUINT; // this will be large enough
 			cycle_size[i] = 0;
+		} else if (is_cyclic[i]) {
+			// FIXME
+			length[i] = cycle_pos[i] = G_MAXUINT; // this will be large enough
+			cycle_size[i] = 0;
+			cyclic_bonds++;
+			if (is_cyclic[i] > 1) {
+				std::list < gcu::Cycle * >::iterator c;
+				gcu::Cycle *c0, *c1;
+				c0 = bond[i]->GetFirstCycle (c, NULL);
+				c1 = bond[i]->GetNextCycle (c, NULL);
+				// Note: we might look for a third cycle for complex situations
+				if (c0->GetLength () > 4 && c1->GetLength () > 4) {
+					// we need to do something if the bridge is 2 bonds or more long.
+					unsigned l = c0->GetBridgeLength (c1, this, bond[i]);
+					if (l > 1) {
+						// we can use stereobonds in this bridge
+						// FIXME: check that it does not contain chiral atoms
+						length[i] = cycle_pos[i] = 0; 
+					}
+				}
+				// with smaller cycles, InChI generation might miss the stereochemistry: FIXME
+			}
 		} else {
 			gcu::Chain *chain = new gcu::Chain (bond[i], this);
 			// find the longuest linear chain and the first attached cycle
