@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
- * molecule.cc 
+ * molecule.cc
  *
  * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -28,9 +28,9 @@
 #include "bond.h"
 #include "document.h"
 #include "molecule.h"
-#include "stringdlg.h"
 #include "tool.h"
 #include "view.h"
+#include <gcugtk/stringdlg.h>
 #include <gcugtk/ui-manager.h>
 #include <gcu/chain.h>
 #include <gsf/gsf-input-memory.h>
@@ -49,12 +49,6 @@ using namespace std;
 
 namespace gcp {
 
-typedef struct {
-	std::string name;
-	std::string uri;
-	std::string classname;
-} BaseAccess;
-
 class MoleculePrivate
 {
 public:
@@ -65,30 +59,24 @@ public:
 	static void ExportTo3D (Molecule *mol);
 	static void ExportToAvogadro (Molecule *mol);
 	static char *Build3D (Molecule *mol);
-	static void LoadDatabases (char const *filename);
-	static void ShowDatabase (GObject *action);
-
-	static std::vector < BaseAccess > Databases;
 };
 
-std::vector < BaseAccess > MoleculePrivate::Databases;
-
 void MoleculePrivate::ShowInChIKey (Molecule *mol)
 {
 
-	new StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetInChIKey (), StringDlg::INCHIKEY);
+	new gcugtk::StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetInChIKey (), gcugtk::StringDlg::INCHIKEY);
 }
 
 void MoleculePrivate::ShowInChI (Molecule *mol)
 {
 
-	new StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetInChI (), StringDlg::INCHI);
+	new gcugtk::StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetInChI (), gcugtk::StringDlg::INCHI);
 }
 
 void MoleculePrivate::ShowSMILES (Molecule *mol)
 {
 
-	new StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetSMILES (), StringDlg::SMILES);
+	new gcugtk::StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetSMILES (), gcugtk::StringDlg::SMILES);
 }
 
 char *MoleculePrivate::Build3D (Molecule *mol)
@@ -112,7 +100,7 @@ void MoleculePrivate::ExportToGhemical (Molecule *mol)
 	uri += tmpname;
 	mol->GetDocument ()->GetApp ()->ConvertFromCML (cml, uri, "gpr");
 	g_free (cml);
-	char *command_line = g_strconcat ("ghemical -f ", tmpname, NULL);	
+	char *command_line = g_strconcat ("ghemical -f ", tmpname, NULL);
 	g_free (tmpname);
 	g_spawn_command_line_async (command_line, NULL);
 	g_free (command_line);
@@ -128,7 +116,7 @@ void MoleculePrivate::ExportTo3D (Molecule *mol)
 	write (f, cml, strlen (cml));
 	close (f);
 	g_free (cml);
-	char *command_line = g_strconcat ("gchem3d-"GCU_API_VER" ", tmpname, NULL);	
+	char *command_line = g_strconcat ("gchem3d-"GCU_API_VER" ", tmpname, NULL);
 	g_free (tmpname);
 	g_spawn_command_line_async (command_line, NULL);
 	g_free (command_line);
@@ -144,103 +132,12 @@ void MoleculePrivate::ExportToAvogadro (Molecule *mol)
 	write (f, cml, strlen (cml));
 	close (f);
 	g_free (cml);
-	char *command_line = g_strconcat ("avogadro ", tmpname, NULL);	
+	char *command_line = g_strconcat ("avogadro ", tmpname, NULL);
 	g_free (tmpname);
 	g_spawn_command_line_async (command_line, NULL);
 	g_free (command_line);
 }
 
-static void
-database_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
-{
-	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
-	if (state->classname == "molecule" && state->name.length () && state->uri.length ())
-		MoleculePrivate::Databases.push_back (*state);
-	state->name.clear ();
-	state->uri.clear ();
-	state->classname.clear ();
-};
-
-static void
-database_name_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
-{
-	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
-	state->name = _(xin->content->str); // this one might be translated
-};
-
-static void
-database_uri_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
-{
-	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
-	state->uri = xin->content->str;
-};
-
-static void
-database_class_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
-{
-	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
-	state->classname = xin->content->str;
-};
-
-static GsfXMLInNode const databases_dtd[] = {
-GSF_XML_IN_NODE (DATABASES, DATABASES, -1, "databases", GSF_XML_NO_CONTENT, NULL, NULL),
-	GSF_XML_IN_NODE (DATABASES, DATABASE, -1, "database", GSF_XML_NO_CONTENT, NULL, database_end),
-		GSF_XML_IN_NODE (DATABASE, NAME, -1, "name", GSF_XML_CONTENT, NULL, database_name_end),
-		GSF_XML_IN_NODE (DATABASE, URI, -1, "uri", GSF_XML_CONTENT, NULL, database_uri_end),
-		GSF_XML_IN_NODE (DATABASE, CLASS, -1, "class", GSF_XML_CONTENT, NULL, database_class_end),
-	GSF_XML_IN_NODE_END
-};
-
-void MoleculePrivate::LoadDatabases (char const *filename)
-{
-	GError *err = NULL;
-	GsfInput *input = gsf_input_stdio_new (filename, &err);
-	if (err) {
-		g_error_free (err);
-		return;
-	}
-	GsfXMLInDoc *xml = gsf_xml_in_doc_new (databases_dtd, NULL);
-	BaseAccess state;
-	gsf_xml_in_doc_parse (xml, input, &state);
-
-	gsf_xml_in_doc_free (xml);
-	g_object_unref (input);
-}
-
-typedef struct {
-	Molecule *mol;
-	BaseAccess *access;
-} AccessState;
-
-void MoleculePrivate::ShowDatabase (GObject *action)
-{
-	AccessState *state = static_cast < AccessState * > (g_object_get_data (action, "state"));
-	if (state) {
-		std::string uri = state->access->uri;
-		size_t pos = uri.find ('%');
-		std::string key;
-		switch (uri[pos + 1]) {
-		case 'I':
-			key = state->mol->GetInChI ();
-			break;
-		case 'K':
-			key = state->mol->GetInChIKey ();
-			break;
-		case 'S':
-			key = state->mol->GetSMILES ();
-			break;
-		default:
-			return;
-		}
-		if (key.length () == 0)
-			return;
-		char *escaped = g_uri_escape_string (key.c_str (), NULL, false);
-		uri.replace (pos, 2, escaped);
-		g_free (escaped);
-		static_cast < Document * > (state->mol->GetDocument ())->GetApplication ()->ShowURI (uri);
-	}
-}
-
 static void do_select_alignment (GObject *action, Molecule* pMol)
 {
 	Object *object = (Object*) g_object_get_data (action, "item");
@@ -252,13 +149,13 @@ static void do_open_in_calc (Molecule* pMol)
 	pMol->OpenCalc ();
 }
 
-Molecule::Molecule (TypeId Type): gcu::Molecule (Type, gcu::ContentType2D)
+Molecule::Molecule (TypeId Type): gcugtk::Molecule (Type, gcu::ContentType2D)
 {
 	m_Alignment = NULL;
 	m_IsResidue = false;
 }
 
-Molecule::Molecule (Atom* pAtom): gcu::Molecule (pAtom, gcu::ContentType2D)
+Molecule::Molecule (Atom* pAtom): gcugtk::Molecule (pAtom, gcu::ContentType2D)
 {
 	m_Alignment = NULL;
 	m_IsResidue = false;
@@ -302,7 +199,7 @@ void Molecule::AddFragment (Fragment* pFragment)
 	m_Fragments.remove (pFragment); // just in case
 	m_Fragments.push_back (pFragment);
 	Object::AddChild (pFragment);
-} 
+}
 
 void Molecule::AddBond (gcu::Bond* pBond)
 {
@@ -348,9 +245,9 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 		bool DoMerge;
 		int n;
 		map<Atom*, MergedAtom*> AtomMap;
-		map<Atom*, MergedAtom*>::iterator j, endj; 
+		map<Atom*, MergedAtom*>::iterator j, endj;
 		map<Bond*, Bond*> BondMap;
-		map<Bond*, Bond*>::iterator b, endb; 
+		map<Bond*, Bond*>::iterator b, endb;
 		for (; i != end; i++) {
 			(*i)->GetCoords (&x, &y);
 			pAtom = (Atom*) pMolecule->GetAtomAt (x, y);
@@ -419,7 +316,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 			for (j = AtomMap.begin (); j != endj; j++) {
 				(*j).first->GetCoords (&x0, &y0);
 				(*j).second->pAtom->GetCoords (&x1, &y1);
-				(*j).first->Move ((x1 - x0) / 2.,(y1 - y0) / 2.); 
+				(*j).first->Move ((x1 - x0) / 2.,(y1 - y0) / 2.);
 			}
 			View *pView = ((Document*) GetDocument ())->GetView ();
 
@@ -443,7 +340,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 			}
 
 			// Treat shared atoms and delete from pMolecule
-			map< gcu::Atom *, gcu::Bond * >::iterator ai;  	 
+			map< gcu::Atom *, gcu::Bond * >::iterator ai;
 			endj = AtomMap.end ();
 			for (j = AtomMap.begin (); j != endj; j++) {
 				b0 = (Bond*) (*j).second->pAtom->GetFirstBond (ai);
@@ -455,7 +352,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 				pMolecule->Remove ((*j).second->pAtom);
 				pView->Remove ((*j).second->pAtom);
 				delete (*j).second->pAtom;
-			}			
+			}
 
 			// Try to restore max bond order for shared bonds and destroy old bonds
 			endb = BondMap.end ();
@@ -543,7 +440,7 @@ bool Molecule::Load (xmlNodePtr node)
 	// FIXME, the following looks like a kludge
 	child = GetNodeByName (node, "pseudo-atom");
 	while (child) {
-		pObject = CreateObject ("pseudo-atom", pDoc);
+		pObject = (GetApplication ())? CreateObject ("pseudo-atom", pDoc): Application::GetApplication ("GChemPaint")->CreateObject ("pseudo-atom", pDoc);
 		if (pDoc)
 			AddChild (pObject);
 		if (!pObject->Load (child)) {
@@ -575,8 +472,8 @@ bool Molecule::Load (xmlNodePtr node)
 		pObject = new Bond ();
 		AddBond ((Bond*) pObject);
 		if (!pObject->Load (child)) {
-			delete pObject;
 			m_Bonds.remove ((Bond*) pObject);
+			delete pObject;
 			return false;
 		}
 		if (pDoc)
@@ -644,7 +541,7 @@ Object* Molecule::GetAtomAt (double x, double y, G_GNUC_UNUSED double z)
 			break;
 	return pObj;
 }
-	
+
 double Molecule::GetYAlign ()
 {
 	if (m_Alignment)
@@ -708,48 +605,17 @@ bool Molecule::BuildContextualMenu (gcu::UIManager *UIManager, Object *object, d
 				gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Molecule'><menu action='open3d'><menuitem action='avogadro'/></menu></menu></popup></ui>", -1, NULL);
 			}
 		}
-		// add databases submenu		
-		if (MoleculePrivate::Databases.empty ()) {
-			// load them using gsf xml support
-			// first $DATADIR/gchemutils/API_VER/databases.xml
-			MoleculePrivate::LoadDatabases (DATADIR"/gchemutils/"GCU_API_VER"/databases.xml");
-			// and now $HOME/.gchemutils/datatases.xml
-			std::string home = getenv ("HOME");
-			home += "/.gchemutils/databases.xml";
-			MoleculePrivate::LoadDatabases (home.c_str ());
-		}
-		if (!MoleculePrivate::Databases.empty ()) {
-			action = gtk_action_new ("database", _("Find in databases"), NULL, NULL);
-			gtk_action_group_add_action (group, action);
-			g_object_unref (action);
-			std::vector < BaseAccess >::iterator it, end = MoleculePrivate::Databases.end ();
-			for (it = MoleculePrivate::Databases.begin (); it != end; it++) {
-				AccessState *state = g_new0 (AccessState, 1);
-				state->access = &(*it);
-				state->mol = this;
-				action = gtk_action_new ((*it).name.c_str (), (*it).name.c_str (), NULL, NULL);
-				g_object_set_data_full (G_OBJECT (action), "state", state, g_free);
-				g_signal_connect (action, "activate", G_CALLBACK (MoleculePrivate::ShowDatabase), NULL);
-				gtk_action_group_add_action (group, action);
-				g_object_unref (action);
-				std::string node = "<ui><popup><menu action='Molecule'><menu action='database'><menuitem action='";
-				node += (*it).name;
-				node += "'/></menu></menu></popup></ui>";
-				gtk_ui_manager_add_ui_from_string (uim, node.c_str (), -1, NULL);
-			}
-		}
-		//		if (((Document*) GetDocument ())->GetApplication ()->HaveInChI ()) {
-			action = gtk_action_new ("inchi", _("Generate InChI"), NULL, NULL);
-			g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowInChI), this);
-			gtk_action_group_add_action (group, action);
-			g_object_unref (action);
-			gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Molecule'><menuitem action='inchi'/></menu></popup></ui>", -1, NULL);
-			action = gtk_action_new ("inchikey", _("Generate InChIKey"), NULL, NULL);
-			g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowInChIKey), this);
-			gtk_action_group_add_action (group, action);
-			g_object_unref (action);
-			gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Molecule'><menuitem action='inchikey'/></menu></popup></ui>", -1, NULL);
-//		}
+		BuildDatabasesMenu (uim, "<ui><popup><menu action='Molecule'>", "</menu></popup></ui>");
+		action = gtk_action_new ("inchi", _("Generate InChI"), NULL, NULL);
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowInChI), this);
+		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
+		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Molecule'><menuitem action='inchi'/></menu></popup></ui>", -1, NULL);
+		action = gtk_action_new ("inchikey", _("Generate InChIKey"), NULL, NULL);
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowInChIKey), this);
+		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
+		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Molecule'><menuitem action='inchikey'/></menu></popup></ui>", -1, NULL);
 		action = gtk_action_new ("smiles", _("Generate SMILES"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowSMILES), this);
 		gtk_action_group_add_action (group, action);
@@ -860,8 +726,73 @@ std::string Molecule::GetRawFormula () const
 	return ofs.str ();
 }
 
+static void BuildConnectivity (gcu::Atom *atom, set < gcu::Atom * > &ConnectedAtoms)
+{
+	std::map < gcu::Atom *, gcu::Bond * >::iterator i;
+	gcu::Bond *bond;
+	ConnectedAtoms.insert (atom);
+	for (bond = atom->GetFirstBond (i); bond; bond = atom->GetNextBond (i)) {
+		gcu::Atom *end = bond->GetAtom (atom);
+		if (ConnectedAtoms.find (end) == ConnectedAtoms.end ())
+			BuildConnectivity (end, ConnectedAtoms);
+	}
+}
+
 void Molecule::OnLoaded ()
 {
+	// First update fragments;
+	std::list < Fragment * >::iterator f, fend = m_Fragments.end ();
+	for (f = m_Fragments.begin (); f != fend; f++)
+		(*f)->Update ();
+	// Then check if we have only one molecule or split, this might happen with
+	// cml (root node being molecule) or malformed files.
+	if (m_Atoms.size () + m_Fragments.size () > 1) {
+		/* FIXME: what should be done with fragments? */
+		std::set < gcu::Atom * > ConnectedAtoms;
+		std::list < gcu::Atom * >:: iterator i;
+		while (1) {
+			if (m_Atoms.size () > 0)
+				BuildConnectivity (GetFirstAtom (i), ConnectedAtoms);
+			else if (m_Fragments.size () > 0) {
+				BuildConnectivity ((*m_Fragments.begin())->GetAtom (), ConnectedAtoms);
+			} else
+				break;
+			if (m_Atoms.size () + m_Fragments.size () == ConnectedAtoms.size ())
+				break;
+			// now split the molecule
+			Molecule *new_mol;
+			Atom *atom = (m_Atoms.size ())?
+							static_cast < Atom * > (GetFirstAtom (i)):
+							(*m_Fragments.begin())->GetAtom ();
+			new_mol = new Molecule ();
+			GetParent ()->AddChild (new_mol);
+			new_mol->AddChild (atom);
+			gcu::Chain *chain = new gcu::Chain (new_mol, atom);
+			delete chain;
+			// move chiral atoms to new mol if needed
+			std::set < gcu::Atom * >::iterator i, iend = ConnectedAtoms.end ();
+			for (i = ConnectedAtoms.begin (); i != iend; i++) {
+				Atom *atom = static_cast < Atom * > (*i);
+				if (m_ChiralAtoms.find (atom) != m_ChiralAtoms.end ()) {
+					m_ChiralAtoms.erase (atom);
+					new_mol->m_ChiralAtoms.insert (atom);
+				}
+			}
+			// remove new_mol children from there
+			std::list < gcu::Atom * >::iterator a, aend = new_mol->m_Atoms.end ();
+			std::list < gcu::Bond * >::iterator b, bend = new_mol->m_Bonds.end ();
+			fend = new_mol->m_Fragments.end ();
+			// update fragments if needed
+			for (a = new_mol->m_Atoms.begin (); a != aend; a++)
+				m_Atoms.remove (*a);
+			for (b = new_mol->m_Bonds.begin (); b != bend; b++)
+				m_Bonds.remove (*b);
+			for (f = new_mol->m_Fragments.begin (); f != fend; f++)
+				m_Fragments.remove (*f);
+			// clean connected atoms for the next round
+			ConnectedAtoms.clear ();
+		}
+	}
 	UpdateCycles ();
 	// now we need to consider atoms with parity (essentially on import)
 	std::set < Atom * >::iterator it, end = m_ChiralAtoms.end ();
@@ -873,6 +804,13 @@ void Molecule::OnLoaded ()
 	// for now, we don't keep them
 	for (it = done.begin (); it != end; it++)
 		m_ChiralAtoms.erase (*it);
+	// check for crossing bonds
+	std::list < gcu::Bond * >::const_iterator i;
+	Bond const *bond = static_cast < Bond const * > (GetFirstBond (i));
+	while (bond) {
+		CheckCrossings (const_cast <Bond *> (bond));
+		bond = static_cast < Bond const * > (GetNextBond (i));
+	}
 	GetParent ()->OnLoaded (); // FIXME: shouldn't needed.
 }
 
@@ -895,7 +833,7 @@ double Molecule::GetMeanBondLength () const
 
 bool Molecule::AtomIsChiral (Atom *atom) const {
 	std::set < Atom * >::const_iterator it = m_ChiralAtoms.find (atom);
-	if (it != m_ChiralAtoms.end ())
+	if (it == m_ChiralAtoms.end ())
 		return false;
 	return atom->HasStereoBond ();
 }
