@@ -256,9 +256,13 @@ static void on_about (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED Chem3dWindo
 static void on_recent (GtkRecentChooser *widget, Chem3dWindow *Win)
 {
 	Chem3dApplication *App = static_cast < Chem3dApplication * > (Win->GetApplication ());
+	Chem3dDoc *doc = Win->GetDocument ();
 	GtkRecentInfo *info = gtk_recent_chooser_get_current_item (widget);
-	App->FileProcess (gtk_recent_info_get_uri (info), gtk_recent_info_get_mime_type (info), false, NULL, Win->GetDocument ());
+	App->FileProcess (gtk_recent_info_get_uri (info), gtk_recent_info_get_mime_type (info), false, NULL, doc);
 	gtk_recent_info_unref(info);
+	Molecule *mol = static_cast < gcugtk::Molecule * > (doc->GetMol ());
+	if (mol && mol->GetChildrenNumber ())
+		Win->AddMoleculeMenus (mol);
 }
 
 static GtkActionEntry entries[] = {
@@ -281,7 +285,7 @@ static GtkActionEntry entries[] = {
 		  N_("Quit GChem3D"), G_CALLBACK (on_quit) },
   { "ViewMenu", NULL, N_("_View"), NULL, NULL, NULL },
   { "ToolsMenu", NULL, N_("_Tools"), NULL, NULL, NULL },
-	  { "ImportMol", NULL, N_("_Import molecule"), NULL,
+	  { "ImportMol", NULL, N_("_Import molecule..."), NULL,
 		  N_("Import a molecule either from InChI or SMILES"), G_CALLBACK (Chem3dWindowPrivate::ImportMolecule) },
 	  { "GChemPaint", NULL, N_("Open in GChemPaint"), NULL,
 		  N_("Open a 2D model for this molecule using GChemPaint"), G_CALLBACK (Chem3dWindowPrivate::OnOpen2D) },
@@ -398,7 +402,6 @@ m_View (NULL)
 	if (!gtk_ui_manager_add_ui_from_string (manager, ui_description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
-		exit (EXIT_FAILURE);
 	}
 	if (extra_ui && !gtk_ui_manager_add_ui_from_string (manager, extra_ui, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
